//----------------------------------------------------------------------
//
//   http://ariva.sat-elita.net/
//
//----------------------------------------------------------------------
#include <stdio.h>
//----------------------------------------------------------------------
#define LOGOPOS 0x00230000
//----------------------------------------------------------------------
#define ABSSIZE 0x00400000
#define LOGOSIZE 0x00010000
//----------------------------------------------------------------------
#define NCRC 0x4E435243
//----------------------------------------------------------------------
unsigned char ABS[ABSSIZE];
unsigned char LOGO[LOGOSIZE];
//----------------------------------------------------------------------
unsigned int ArivaCRC(const unsigned char *data, unsigned int length)
{
unsigned int i,j;   
unsigned int crc=0xFFFFFFFF;   
for (i=0; i<length; i++)
   {   
   crc^=((unsigned int)(data[i])<<24);   
   for (j=0; j<8; j++)   
      {   
      if (crc & 0x80000000) {crc=((crc<<1)^0x04c11db7);}
      else {crc<<=1;};   
      };   
   };   
return crc;   
}
//----------------------------------------------------------------------
unsigned int BigEndInt(const unsigned char *data)
{
unsigned int ret;
ret=(unsigned int)(data[0]); ret<<=8;
ret|=(unsigned int)(data[1]); ret<<=8;
ret|=(unsigned int)(data[2]); ret<<=8;
ret|=(unsigned int)(data[3]);
return ret;
}
//----------------------------------------------------------------------
void PutInt(unsigned char *data, unsigned int i)
{
data[0]=(unsigned char)(i>>24 & 0xff);	
data[1]=(unsigned char)(i>>16 & 0xff);	
data[2]=(unsigned char)(i>>8 & 0xff);	
data[3]=(unsigned char)(i & 0xff);	
}
//----------------------------------------------------------------------
void ArivaModulCRC(unsigned char *a, unsigned int pos)
{
unsigned int c,u,s,i;
char *abs=&a[pos];
printf("Module (0x%x):\n ID = 0x%X\n",pos,BigEndInt(&abs[0]));
s=BigEndInt(&abs[4]);
i=BigEndInt(&abs[8]);
if ((s==0) && (i==0x00010000)) {s=i;i=0;};
printf(" Size = %i (bytes)\n",s);
printf(" Unknown = 0x%X\n",i);
u=BigEndInt(&abs[12]);
if (u==NCRC) {printf(" CRC = NCRC\n");}
else
   {
   printf(" CRC = 0x%X",u);
   c=ArivaCRC(&abs[0x10],s);
   if (u==c) {printf("\n");}
   else 
      {
      printf(" -> 0x%X\n",c);
      PutInt(&abs[12],c);
      };
   }
printf(" Name = %s\n",&abs[0x10]);
printf(" Ver = %s\n",&abs[0x20]);
printf(" Date = %s\n",&abs[0x30]);
}
//----------------------------------------------------------------------
void InsertLogo(unsigned char *abs, unsigned char *logo, unsigned int lsize)
{
unsigned int i,olds;
olds=BigEndInt(&abs[LOGOPOS+4]);
if (lsize<olds)
   {
   printf("Copy %i bytes\n",lsize);
   for (i=0; i<lsize; i++) {abs[LOGOPOS+0x80+i]=logo[i];};
   for (i=lsize; i<olds; i++) {abs[LOGOPOS+0x80+i]=0x00;};  
   }
else
   {
   printf("Cut MPG to %i bytes (lost %i bytes)\n",olds,lsize-olds);
   for (i=0; i<olds; i++) {abs[LOGOPOS+0x80+i]=logo[i];};
   };
ArivaModulCRC(ABS,LOGOPOS);
}
//----------------------------------------------------------------------
int LoadLogo(char *lname)
{
int i,ret=0;
FILE *fp;
size_t s;
fp=fopen(lname,"rb");
if (fp)
   {
   for (i=0; i<LOGOSIZE; i++) {LOGO[i]=0xFF;};
   s=fread(LOGO,1,LOGOSIZE,fp); 
   printf("Load Logo \"%s\"(%ibytes)\n",lname,s); 
   ret=s;
   };
return ret;	
}
//----------------------------------------------------------------------
int LoadABS(char *inname)
{
int i,ret=0;
FILE *fp;
size_t s;
fp=fopen(inname,"rb");
if (fp)
   {
   for (i=0; i<ABSSIZE; i++) {ABS[i]=0;};
   s=fread(ABS,1,ABSSIZE,fp);  
   if (s==ABSSIZE) 
      {
      printf("Load ABS \"%s\"\n",inname);
      ret=1;
      };
   };
return ret;	
}
//----------------------------------------------------------------------
int SaveABS(char *outname)
{
int ret=0;
FILE *fp;
size_t s;
fp=fopen(outname,"wb");
if (fp)
   {
   s=fwrite(ABS,1,ABSSIZE,fp);  
   if (s==ABSSIZE) 
      {
      printf("Save ABS \"%s\"\n",outname);
      ret=1;
      };
   };
return ret;	
}
//----------------------------------------------------------------------
int main(int argc, char *argv[])
{
int i,ls;
printf("ArivaLogo\nver: %s\n",__DATE__);
printf("http://ariva.sat-elita.net/\n");
if (argc==4)
   {
   if (LoadABS(argv[1]))
      {
      ls=LoadLogo(argv[2]);
      if (ls)
         {
         InsertLogo(ABS,LOGO,ls);
         if (SaveABS(argv[3])) {printf("Done.\n");}
         else {printf("Save Error: \"%s\"\n",argv[2]);};
         }
      else
         {
         printf("Load Error: \"%s\"\n",argv[2]);		  
         };	   
      }
   else
      {
      printf("Load Error: \"%s\"\n",argv[1]);		  
      };	   
   }
else
   {
   printf("Use: arivalogo filein.abs logo.mpg fileout.abs\n");
   };
return 0;
}
//----------------------------------------------------------------------
